@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2005-2007 Remon Sijrier
+Copyright (C) 2005-2008 Remon Sijrier
 
 This file is part of Traverso
 
@@ -27,7 +27,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include "AudioClip.h"
 #include "AudioSource.h"
 #include "WriteSource.h"
-#include "Song.h"
+#include "Sheet.h"
 #include "SnapList.h"
 #include "Track.h"
 #include "AudioChannel.h"
@@ -50,6 +50,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #include <Config.h>
 #include "PluginChain.h"
 #include "GainEnvelope.h"
+#include "InputEngine.h"
 
 #include "AbstractAudioReader.h"
 
@@ -59,24 +60,27 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 // in case we run with memory leak detection enabled!
 #include "Debugger.h"
 
+/**
+ *	\class AudioClip
+	\brief Represents (part of) an audiofile.
+ */
 
 AudioClip::AudioClip(const QString& name)
-	: ContextItem()
-	, Snappable()
+	: Snappable()
 	, m_name(name)
 {
 	PENTERCONS;
 	m_isMuted=false;
 	m_id = create_id();
-	m_readSourceId = m_songId = 0;
+	m_readSourceId = m_sheetId = 0;
 	init();
 }
 
 
 AudioClip::AudioClip(const QDomNode& node)
-	: ContextItem()
-	, Snappable()
+	: Snappable()
 	, m_track(0)
+	, m_sheet(0)
 	, m_readSource(0)
 {
 	PENTERCONS;
@@ -87,7 +91,7 @@ AudioClip::AudioClip(const QDomNode& node)
 	QDomElement e = node.toElement();
 	m_id = e.attribute("id", "").toLongLong();
 	m_readSourceId = e.attribute("source", "").toLongLong();
-	m_songId = e.attribute("sheet", "0").toLongLong();
+	m_sheetId = e.attribute("sheet", "0").toLongLong();
 	m_name = e.attribute( "clipname", "" ) ;
 	m_isMuted =  e.attribute( "mute", "" ).toInt();
 	// FIXME!!!!!!!
@@ -106,7 +110,7 @@ AudioClip::~AudioClip()
 {
 	PENTERDES;
 	if (m_readSource) {
-		m_song->get_diskio()->unregister_read_source(m_readSource);
+		m_sheet->get_diskio()->unregister_read_source(m_readSource);
 		delete m_readSource;
 	}
 	if (m_peak) {
@@ -116,12 +120,13 @@ AudioClip::~AudioClip()
 
 void AudioClip::init()
 {
-	m_song = 0;
+	QObject::tr("AudioClip");
+	m_sheet = 0;
 	m_track = 0;
 	m_readSource = 0;
 	m_peak = 0;
 	m_recordingStatus = NO_RECORDING;
-	m_isSelected = m_isReadSourceValid = false;
+	m_isReadSourceValid = m_isMoving = false;
 	m_isLocked = config().get_property("AudioClip", "LockByDefault", false).toBool();
 	fadeIn = 0;
 	fadeOut = 0;
@@ -135,20 +140,15 @@ int AudioClip::set_state(const QDomNode& node)
 {
 	PENTER;
 	
-	Q_ASSERT(m_song);
+	Q_ASSERT(m_sheet);
 	
 	QDomElement e = node.toElement();
 
 	m_isTake = e.attribute( "take", "").toInt();
 	set_gain( e.attribute( "gain", "" ).toFloat() );
 	m_isLocked = e.attribute( "locked", "0" ).toInt();
-
-	if (e.attribute("selected", "0").toInt() == 1) {
-		m_song->get_audioclip_manager()->select_clip(this);
-	}
-
 	m_readSourceId = e.attribute("source", "").toLongLong();
-	m_songId = e.attribute("sheet", "0").toLongLong();
+	m_sheetId = e.attribute("sheet", "0").toLongLong();
 	m_isMuted =  e.attribute( "mute", "" ).toInt();
 
 	bool ok;
@@ -163,7 +163,7 @@ int AudioClip::set_state(const QDomNode& node)
 	QDomElement fadeInNode = node.firstChildElement("FadeIn");
 	if (!fadeInNode.isNull()) {
 		if (!fadeIn) {
-			fadeIn = new FadeCurve(this, m_song, "FadeIn");
+			fadeIn = new FadeCurve(this, m_sheet, "FadeIn");
 			fadeIn->set_history_stack(get_history_stack());
 			private_add_fade(fadeIn);
 		}
@@ -173,7 +173,7 @@ int AudioClip::set_state(const QDomNode& node)
 	QDomElement fadeOutNode = node.firstChildElement("FadeOut");
 	if (!fadeOutNode.isNull()) {
 		if (!fadeOut) {
-			fadeOut = new FadeCurve(this, m_song, "FadeOut");
+			fadeOut = new FadeCurve(this, m_sheet, "FadeOut");
 			fadeOut->set_history_stack(get_history_stack());
 			private_add_fade(fadeOut);
 		}
@@ -197,9 +197,8 @@ QDomNode AudioClip::get_state( QDomDocument doc )
 	node.setAttribute("mute", m_isMuted);
 	node.setAttribute("take", m_isTake);
 	node.setAttribute("clipname", m_name );
-	node.setAttribute("selected", m_isSelected );
 	node.setAttribute("id", m_id );
-	node.setAttribute("sheet", m_songId );
+	node.setAttribute("sheet", m_sheetId );
 	node.setAttribute("locked", m_isLocked);
 
 	node.setAttribute("source", m_readSourceId);
@@ -247,7 +246,17 @@ void AudioClip::set_sources_active_state()
 		return;
 	}
 	
-	if ( m_track->is_muted() || m_track->is_muted_by_solo() || is_muted() ) {
+	bool syncDuringDrag = config().get_property("AudioClip", "SyncDuringDrag", false).toBool();
+	bool stopSyncDueMove;
+	if (m_isMoving && syncDuringDrag) {
+		stopSyncDueMove = false;
+	} else if (m_isMoving && !syncDuringDrag){
+		stopSyncDueMove = true;
+	} else {
+		stopSyncDueMove = false;
+	}
+		
+	if ( m_track->is_muted() || m_track->is_muted_by_solo() || is_muted() || stopSyncDueMove) {
 		m_readSource->set_active(false);
 	} else {
 		m_readSource->set_active(true);
@@ -271,37 +280,32 @@ void AudioClip::set_left_edge(TimeRef newLeftLocation)
 			movingToLeft = availableTimeLeft;
 		}
 
-		m_trackStartLocation -= movingToLeft;
 		set_source_start_location( m_sourceStartLocation - movingToLeft );
-
+		set_track_start_location(m_trackStartLocation - movingToLeft);
 	} else if (newLeftLocation > m_trackStartLocation) {
 
 		TimeRef availableTimeRight = m_length;
 
 		TimeRef movingToRight = newLeftLocation - m_trackStartLocation;
 
-		if (movingToRight > (availableTimeRight - TimeRef(4, get_rate())) ) {
-			movingToRight = (availableTimeRight - TimeRef(4, get_rate()));
+		if (movingToRight > (availableTimeRight - TimeRef(nframes_t(4), get_rate())) ) {
+			movingToRight = (availableTimeRight - TimeRef(nframes_t(4), get_rate()));
 		}
 
-		m_trackStartLocation += movingToRight;
 		set_source_start_location( m_sourceStartLocation + movingToRight );
-
-	} else {
-		return;
+		set_track_start_location(m_trackStartLocation + movingToRight);
 	}
-
-	emit positionChanged(this);
 }
 
 void AudioClip::set_right_edge(TimeRef newRightLocation)
 {
+	
 	if (newRightLocation < TimeRef()) {
 		newRightLocation = TimeRef();
 	}
 	
 	if (newRightLocation > m_trackEndLocation) {
-
+		
 		TimeRef availableTimeRight = m_sourceLength - m_sourceEndLocation;
 
 		TimeRef movingToRight = newRightLocation - m_trackEndLocation;
@@ -310,8 +314,8 @@ void AudioClip::set_right_edge(TimeRef newRightLocation)
 			movingToRight = availableTimeRight;
 		}
 
-		set_track_end_location( m_trackEndLocation + movingToRight );
 		set_source_end_location( m_sourceEndLocation + movingToRight );
+		set_track_end_location( m_trackEndLocation + movingToRight );
 
 	} else if (newRightLocation < m_trackEndLocation) {
 
@@ -319,18 +323,13 @@ void AudioClip::set_right_edge(TimeRef newRightLocation)
 
 		TimeRef movingToLeft = m_trackEndLocation - newRightLocation;
 
-		if (movingToLeft > availableTimeLeft - TimeRef(4, get_rate())) {
-			movingToLeft = availableTimeLeft - TimeRef(4, get_rate());
+		if (movingToLeft > availableTimeLeft - TimeRef(nframes_t(4), get_rate())) {
+			movingToLeft = availableTimeLeft - TimeRef(nframes_t(4), get_rate());
 		}
 
-		set_track_end_location( m_trackEndLocation - movingToLeft );
 		set_source_end_location( m_sourceEndLocation - movingToLeft);
-
-	} else {
-		return;
+		set_track_end_location( m_trackEndLocation - movingToLeft );
 	}
-
-	emit positionChanged(this);
 }
 
 void AudioClip::set_source_start_location(const TimeRef& location)
@@ -347,19 +346,37 @@ void AudioClip::set_source_end_location(const TimeRef& location)
 
 void AudioClip::set_track_start_location(const TimeRef& location)
 {
+	PENTER2;
 	m_trackStartLocation = location;
+	
+	// set_track_end_location will emit positionChanged(), so we 
+	// don't emit it in this function to avoid emitting it twice
+	// (although it seems more logical to emit it here, there are
+	// accasions where only set_track_end_location() is called, and 
+	// then we also want to emit positionChanged())
 	set_track_end_location(m_trackStartLocation + m_length);
-	if (m_track) {
-		THREAD_SAVE_INVOKE(m_track, this, clip_position_changed(AudioClip*));
-	}
-	emit positionChanged(this);
 }
 
 void AudioClip::set_track_end_location(const TimeRef& location)
 {
 // 	PWARN("m_trackEndLocation is %d", endFrame);
 	m_trackEndLocation = location;
-	emit trackEndLocationChanged();
+	
+	if (!is_moving()) {
+		// TODO find out if we also have to call this even during moving?
+		// And somehow Track itself should manage this, and not from here!
+		// The purpose of this call is to keep the AudioClip list in track 
+		// sorted on the clips track start location.
+		if (m_track) {
+			THREAD_SAVE_INVOKE(m_track, this, clip_position_changed(AudioClip*));
+		}
+		
+		if (m_sheet) {
+			m_sheet->get_snap_list()->mark_dirty();
+		}
+	}
+
+	emit positionChanged();
 }
 
 void AudioClip::set_fade_in(double range)
@@ -392,7 +409,6 @@ void AudioClip::set_gain(float gain)
 
 void AudioClip::set_selected(bool selected)
 {
-	m_isSelected = selected;
 	emit stateChanged();
 }
 
@@ -401,7 +417,12 @@ void AudioClip::set_selected(bool selected)
 //
 int AudioClip::process(nframes_t nframes)
 {
-	Q_ASSERT(m_song);
+	Q_ASSERT(m_sheet);
+	
+	// Handle silence clips
+	if (get_channels() == 0) {
+		return 0;
+	}
 	
 	if (m_recordingStatus == RECORDING) {
 		process_capture(nframes);
@@ -418,7 +439,7 @@ int AudioClip::process(nframes_t nframes)
 	
 	Q_ASSERT(m_readSource);
 	
-	AudioBus* bus = m_song->get_clip_render_bus();
+	AudioBus* bus = m_sheet->get_clip_render_bus();
 	bus->silence_buffers(nframes);
 	
 	TimeRef mix_pos;
@@ -428,7 +449,7 @@ int AudioClip::process(nframes_t nframes)
 
 
 	int outputRate = m_readSource->get_output_rate();
-	TimeRef transportLocation = m_song->get_transport_location();
+	TimeRef transportLocation = m_sheet->get_transport_location();
 	TimeRef upperRange = transportLocation + TimeRef(framesToProcess, outputRate);
 	
 	
@@ -467,13 +488,13 @@ int AudioClip::process(nframes_t nframes)
 
 	uint read_frames = 0;
 
-	if (m_song->realtime_path()) {
+	if (m_sheet->realtime_path()) {
 		read_frames = m_readSource->rb_read(mixdown, mix_pos, framesToProcess);
 	} else {
-		read_frames = m_readSource->file_read(m_song->renderDecodeBuffer, mix_pos, framesToProcess);
+		read_frames = m_readSource->file_read(m_sheet->renderDecodeBuffer, mix_pos, framesToProcess);
 		if (read_frames > 0) {
 			for (int chan=0; chan<channelcount; ++chan) {
-				memcpy(mixdown[chan], m_song->renderDecodeBuffer->destination[chan], read_frames * sizeof(audio_sample_t));
+				memcpy(mixdown[chan], m_sheet->renderDecodeBuffer->destination[chan], read_frames * sizeof(audio_sample_t));
 			}
 		}
 	}
@@ -495,7 +516,7 @@ int AudioClip::process(nframes_t nframes)
 	TimeRef endlocation = mix_pos + TimeRef(read_frames, get_rate());
 	m_fader->process_gain(mixdown, mix_pos, endlocation, read_frames, channelcount);
 	
-	AudioBus* sendbus = m_song->get_render_bus();
+	AudioBus* sendbus = m_sheet->get_render_bus();
 	
 	// NEVER EVER FORGET that the mixing should be done on the WHOLE buffer, not just part of it
 	// so use an unmodified nframes variable!!!!!!!!!!!!!!!!!!!!!!!!!!!1
@@ -544,7 +565,7 @@ void AudioClip::process_capture(nframes_t nframes)
 
 int AudioClip::init_recording( QByteArray name )
 {
-	Q_ASSERT(m_song);
+	Q_ASSERT(m_sheet);
 	Q_ASSERT(m_track);
 	
 	m_captureBusName = name;
@@ -570,7 +591,7 @@ int AudioClip::init_recording( QByteArray name )
 	
 	ReadSource* rs = resources_manager()->create_recording_source(
 				pm().get_project()->get_root_dir() + "/audiosources/",
-				m_name, channelcount, m_song->get_id());
+				m_name, channelcount, m_sheet->get_id());
 	
 	resources_manager()->set_source_for_clip(this, rs);
 	
@@ -618,10 +639,10 @@ int AudioClip::init_recording( QByteArray name )
 	m_recorder->set_process_peaks( true );
 	m_recorder->set_recording( true );
 	
-	m_song->get_diskio()->register_write_source(m_recorder);
+	m_sheet->get_diskio()->register_write_source(m_recorder);
 	
 	connect(m_recorder, SIGNAL(exportFinished()), this, SLOT(finish_write_source()));
-	connect(m_song, SIGNAL(transferStopped()), this, SLOT(finish_recording()));
+	connect(m_sheet, SIGNAL(transportStopped()), this, SLOT(finish_recording()));
 	connect(&audiodevice(), SIGNAL(driverParamsChanged()), this, SLOT(get_capture_bus()));
 
 	return 1;
@@ -659,22 +680,31 @@ Command* AudioClip::reset_fade_out()
 
 Command* AudioClip::reset_fade_both()
 {
-	CommandGroup* group = new CommandGroup(this, tr("Reset Fades"));
-	group->add_command(reset_fade_in());
-	group->add_command(reset_fade_out());
-
+	if (!fadeOut && !fadeIn) {
+		return 0;
+	}
+	
+	CommandGroup* group = new CommandGroup(this, tr("Remove Fades"));
+	
+	if (fadeIn) {
+		group->add_command(reset_fade_in());
+	}
+	if (fadeOut) {
+		group->add_command(reset_fade_out());
+	}
+	
 	return group;
 }
 
 AudioClip* AudioClip::create_copy( )
 {
 	PENTER;
-	Q_ASSERT(m_song);
+	Q_ASSERT(m_sheet);
 	Q_ASSERT(m_track);
 	QDomDocument doc("AudioClip");
 	QDomNode clipState = get_state(doc);
 	AudioClip* clip = new AudioClip(m_name);
-	clip->set_song(m_song);
+	clip->set_sheet(m_sheet);
 	clip->set_track(m_track);
 	clip->set_state(clipState);
 	return clip;
@@ -708,8 +738,6 @@ void AudioClip::set_audio_source(ReadSource* rs)
 		m_length = m_sourceEndLocation;
 	}
 
-	set_track_end_location(m_trackStartLocation + m_sourceLength - m_sourceStartLocation);
-
 	set_sources_active_state();
 
 	rs->set_audio_clip(this);
@@ -724,7 +752,8 @@ void AudioClip::set_audio_source(ReadSource* rs)
 		}
 	}
 
-	emit stateChanged();
+	// This will also emit positionChanged() which is more or less what we want.
+	set_track_end_location(m_trackStartLocation + m_sourceLength - m_sourceStartLocation);
 }
 
 void AudioClip::finish_write_source()
@@ -736,7 +765,7 @@ void AudioClip::finish_write_source()
 	if (m_readSource->set_file(m_recorder->get_filename()) < 0) {
 		PERROR("Setting file for ReadSource failed after finishing recording");
 	} else {
-		m_song->get_diskio()->register_read_source(m_readSource);
+		m_sheet->get_diskio()->register_read_source(m_readSource);
 		// re-inits the lenght from the audiofile due calling rsm->set_source_for_clip()
 		m_length = TimeRef();
 	}
@@ -758,7 +787,7 @@ void AudioClip::finish_recording()
 	m_recordingStatus = FINISHING_RECORDING;
 	m_recorder->set_recording(false);
 
-	disconnect(m_song, SIGNAL(transferStopped()), this, SLOT(finish_recording()));
+	disconnect(m_sheet, SIGNAL(transportStopped()), this, SLOT(finish_recording()));
 	disconnect(&audiodevice(), SIGNAL(driverParamsChanged()), this, SLOT(get_capture_bus()));
 }
 
@@ -775,10 +804,10 @@ int AudioClip::get_channels( ) const
 	return 0;
 }
 
-Song* AudioClip::get_song( ) const
+Sheet* AudioClip::get_sheet( ) const
 {
-	Q_ASSERT(m_song);
-	return m_song;
+	Q_ASSERT(m_sheet);
+	return m_sheet;
 }
 
 Track* AudioClip::get_track( ) const
@@ -787,20 +816,20 @@ Track* AudioClip::get_track( ) const
 	return m_track;
 }
 
-void AudioClip::set_song( Song * song )
+void AudioClip::set_sheet( Sheet * sheet )
 {
-	m_song = song;
+	m_sheet = sheet;
 	if (m_readSource && m_isReadSourceValid) {
-		m_song->get_diskio()->register_read_source( m_readSource );
+		m_sheet->get_diskio()->register_read_source( m_readSource );
 	} else {
-		PWARN("AudioClip::set_song() : Setting Song, but no ReadSource available!!");
+		PWARN("AudioClip::set_sheet() : Setting Sheet, but no ReadSource available!!");
 	}
 	
-	m_songId = song->get_id();
+	m_sheetId = sheet->get_id();
 	
-	set_history_stack(m_song->get_history_stack());
-	m_pluginChain->set_song(m_song);
-	set_snap_list(m_song->get_snap_list());
+	set_history_stack(m_sheet->get_history_stack());
+	m_pluginChain->set_sheet(m_sheet);
+	set_snap_list(m_sheet->get_snap_list());
 }
 
 
@@ -822,9 +851,10 @@ void AudioClip::set_name( const QString& name )
 	emit stateChanged();
 }
 
-bool AudioClip::is_selected( ) const
+bool AudioClip::is_selected( )
 {
-	return m_isSelected;
+	Q_ASSERT(m_sheet);
+	return m_sheet->get_audioclip_manager()->is_clip_in_selection(this);
 }
 
 bool AudioClip::is_take( ) const
@@ -856,16 +886,11 @@ int AudioClip::get_rate( ) const
 	return audiodevice().get_sample_rate();
 }
 
-TimeRef& AudioClip::get_source_length( ) const
+TimeRef AudioClip::get_source_length( ) const
 {
 	return m_sourceLength;
 }
 
-TimeRef& AudioClip::get_length() const
-{
-	return m_length;
-}
-
 int AudioClip::recording_state( ) const
 {
 	return m_recordingStatus;
@@ -874,22 +899,22 @@ int AudioClip::recording_state( ) const
 Command * AudioClip::normalize( )
 {
         bool ok;
+        float normfactor;
         double d = QInputDialog::getDouble(0, tr("Normalization"),
                                            tr("Set Normalization level:"), 0.0, -120, 0, 1, &ok);
         if (ok) {
-		calculate_normalization_factor(d);
-	}
+                normfactor = calculate_normalization_factor(d);
+        }
 
-	return (Command*) 0;
+        if (!ok || (normfactor == get_gain())) {
+            return ie().failure();
+        }
+
+        return new PCommand(this, "set_gain", normfactor, get_gain(), tr("AudioClip: Normalize"));
 }
 
-Command * AudioClip::denormalize( )
-{
-	set_gain(1.0);
-	return (Command*) 0;
-}
 
-void AudioClip::calculate_normalization_factor(float targetdB)
+float AudioClip::calculate_normalization_factor(float targetdB)
 {
 	float target = dB_to_scale_factor (targetdB);
 
@@ -905,17 +930,17 @@ void AudioClip::calculate_normalization_factor(float targetdB)
 	if (maxamp == 0.0f) {
 		printf("AudioClip::normalization: max amplitude == 0\n");
 		/* don't even try */
-		return;
+                return get_gain();
 	}
 	
 	if (maxamp == target) {
 		printf("AudioClip::normalization: max amplitude == target amplitude\n");
 		/* we can't do anything useful */
-		return;
+                return get_gain();
 	}
 
 	/* compute scale factor */
-	set_gain(target/maxamp);
+        return float(target/maxamp);
 }
 
 FadeCurve * AudioClip::get_fade_in( ) const
@@ -952,7 +977,7 @@ void AudioClip::private_remove_fade( FadeCurve * fade )
 
 void AudioClip::create_fade_in( )
 {
-	fadeIn = new FadeCurve(this, m_song, "FadeIn");
+	fadeIn = new FadeCurve(this, m_sheet, "FadeIn");
 	fadeIn->set_shape("Fast");
 	fadeIn->set_history_stack(get_history_stack());
 	THREAD_SAVE_INVOKE_AND_EMIT_SIGNAL(this, fadeIn, private_add_fade(FadeCurve*), fadeAdded(FadeCurve*));
@@ -960,7 +985,7 @@ void AudioClip::create_fade_in( )
 
 void AudioClip::create_fade_out( )
 {
-	fadeOut = new FadeCurve(this, m_song, "FadeOut");
+	fadeOut = new FadeCurve(this, m_sheet, "FadeOut");
 	fadeOut->set_shape("Fast");
 	fadeOut->set_history_stack(get_history_stack());
 	THREAD_SAVE_INVOKE_AND_EMIT_SIGNAL(this, fadeOut, private_add_fade(FadeCurve*), fadeAdded(FadeCurve*));
@@ -971,9 +996,9 @@ QDomNode AudioClip::get_dom_node() const
 	return m_domNode;
 }
 
-bool AudioClip::has_song() const
+bool AudioClip::has_sheet() const
 {
-	if (m_song) {
+	if (m_sheet) {
 		return true;
 	}
 	return false;
@@ -989,3 +1014,10 @@ void AudioClip::get_capture_bus()
 	m_captureBus = audiodevice().get_capture_bus(m_captureBusName);
 }
 
+void AudioClip::set_as_moving(bool moving)
+{
+	m_isMoving = moving;
+	set_snappable(!m_isMoving);
+	set_sources_active_state();
+}
+
