@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2006-2007 Nicola Doebelin, Remon Sijrier
+Copyright (C) 2006-2008 Nicola Doebelin, Remon Sijrier
 
 This file is part of Traverso
 
@@ -20,10 +20,12 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */
 
 #include "SnapList.h"
+
 #include "Peak.h"
-#include "Song.h"
+#include "Sheet.h"
 #include "AudioClip.h"
 #include "AudioClipManager.h"
+#include "Config.h"
 #include "ContextPointer.h"
 #include "TimeLine.h"
 #include "Marker.h"
@@ -37,14 +39,11 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 #if defined(debugsnaplist)
 #define SLPRINT(args...) printf(args)
 #else
-#define SLPRINT(args...)
+#define SLPRINT(args...);
 #endif
 
-static const int SNAP_WIDTH = 10;
-
-SnapList::SnapList(Song* song) 
-	: QObject(song)
-	, m_song(song)
+SnapList::SnapList(Sheet* sheet) 
+	: m_sheet(sheet)
 {
 	m_isDirty = true;
 	m_rangeStart = TimeRef();
@@ -52,11 +51,11 @@ SnapList::SnapList(Song* song)
 	m_scalefactor = 1;
 }
 
-void SnapList::mark_dirty(Snappable *item)
+void SnapList::mark_dirty()
 {
-	if (item->is_snappable()) {
-		m_isDirty = true;
-	}
+// 	printf("mark_dirty()\n");
+	m_isDirty = true;
+	m_wasDirty = true;
 }
 
 void SnapList::update_snaplist()
@@ -66,18 +65,18 @@ void SnapList::update_snaplist()
 	m_xposBool.clear();
 	
 	// collects all clip boundaries and adds them to the snap list
-	QList<AudioClip* >* acList = m_song->get_audioclip_manager()->get_clip_list();
+	QList<AudioClip* > acList = m_sheet->get_audioclip_manager()->get_clip_list();
 	
-	SLPRINT("acList size is %d\n", acList->size());
+	SLPRINT("acList size is %d\n", acList.size());
 
 	// Be able to snap to trackstart
 	if (m_rangeStart == qint64(0)) {
 		m_xposList.append(TimeRef());
 	}
 
-	for( int i = 0; i < acList->size(); i++ ) {
+	for( int i = 0; i < acList.size(); i++ ) {
 
-		AudioClip* clip = acList->at(i);
+		AudioClip* clip = acList.at(i);
 		if ( ! clip->is_snappable()) {
 			continue;
 		}
@@ -98,7 +97,7 @@ void SnapList::update_snaplist()
 	}
 
 	// add all on-screen markers
-	QList<Marker*> markerList = m_song->get_timeline()->get_markers();
+	QList<Marker*> markerList = m_sheet->get_timeline()->get_markers();
 	for (int i = 0; i < markerList.size(); ++i) {
 		if (markerList.at(i)->is_snappable() && markerList.at(i)->get_when() >= m_rangeStart && markerList.at(i)->get_when() <= m_rangeEnd) {
 			m_xposList.append(markerList.at(i)->get_when());
@@ -106,10 +105,10 @@ void SnapList::update_snaplist()
 	}
 
 	// add the working cursor's position
-	TimeRef worklocation = m_song->get_work_location();
+	TimeRef worklocation = m_sheet->get_work_location();
 	//printf("worklocation xpos is %d\n",  worklocation / m_scalefactor);
-	if (m_song->get_work_snap()->is_snappable() && worklocation >= m_rangeStart && worklocation <= m_rangeEnd) {
-		m_xposList.append(m_song->get_work_location());
+	if (m_sheet->get_work_snap()->is_snappable() && worklocation >= m_rangeStart && worklocation <= m_rangeEnd) {
+		m_xposList.append(m_sheet->get_work_location());
 	}
 	
 
@@ -133,16 +132,17 @@ void SnapList::update_snaplist()
 		}
 
 		// check if neighbouring snap regions overlap.
-		// if yes, reduce SNAP_WIDTH to keep the border in the middle
-		int ls = -SNAP_WIDTH;
+		// if yes, reduce snap-range to keep the border in the middle
+		int snaprange = config().get_property("Snap", "range", 10).toInt();
+		int ls = - snaprange;
 
 		if (lastIndex > -1) {
-			if ( (m_xposList.at(i) - lastVal) < (2 * SNAP_WIDTH * m_scalefactor) ) {
+			if ( (m_xposList.at(i) - lastVal) < (2 * snaprange * m_scalefactor) ) {
 				ls = - (int) ((m_xposList.at(i) / m_scalefactor - lastVal / m_scalefactor) / 2);
 			}
 		}
 
-		for (int j = ls; j <= SNAP_WIDTH; j++) {
+		for (int j = ls; j <= snaprange; j++) {
 			int pos = (int)((m_xposList.at(i) - m_rangeStart) / m_scalefactor + j); // index in the LUT
 
 			if (pos < 0) {
@@ -166,7 +166,7 @@ void SnapList::update_snaplist()
 
 
 // public function that checks if there is a snap position
-// within +-SNAP_WIDTH of the supplied value i
+// within +- snap-range of the supplied value i
 TimeRef SnapList::get_snap_value(const TimeRef& pos)
 {
 	if (m_isDirty) {
@@ -336,3 +336,51 @@ TimeRef SnapList::prev_snap_pos(const TimeRef& pos)
 	return newpos;
 }
 
+
+TimeRef SnapList::calculate_snap_diff(TimeRef leftlocation, TimeRef rightlocation)
+{
+	// "nframe_t" domain, but must be signed ints because they can become negative
+	qint64 snapStartDiff = 0;
+	qint64 snapEndDiff = 0;
+	qint64 snapDiff = 0;
+	
+	// check if there is anything to snap
+	bool start_snapped = false;
+	bool end_snapped = false;
+	if (is_snap_value(leftlocation)) {
+		start_snapped = true;
+	}
+	
+	if (is_snap_value(rightlocation)) {
+		end_snapped = true;
+	}
+
+	if (start_snapped) {
+		snapStartDiff = get_snap_diff(leftlocation);
+		snapDiff = snapStartDiff; // in case both ends snapped, change this value later, else leave it
+	}
+
+	if (end_snapped) {
+		snapEndDiff = get_snap_diff(rightlocation); 
+		snapDiff = snapEndDiff; // in case both ends snapped, change this value later, else leave it
+	}
+
+	// If both snapped, check which one is closer. Do not apply this check if one of the
+	// ends hasn't snapped, because it's diff value will be 0 by default and will always
+	// be smaller than the actually snapped value.
+	if (start_snapped && end_snapped) {
+		if (abs(snapEndDiff) > abs(snapStartDiff))
+			snapDiff = snapStartDiff;
+		else
+			snapDiff = snapEndDiff;
+	}
+	
+	return TimeRef(snapDiff);
+}
+
+bool SnapList::was_dirty()
+{
+	bool ret = m_wasDirty;
+	m_wasDirty = false;
+	return ret;
+}
